@@ -5,6 +5,7 @@ from collections import defaultdict, OrderedDict
 import pyvex
 from archinfo.arch_arm import is_arm_arch
 
+from ....concretization_strategies import SimConcretizationStrategyAny
 from ....knowledge_plugins.cfg import IndirectJump, IndirectJumpType
 from ....engines.vex.claripy import ccall
 from ....engines.light import SimEngineLightVEXMixin, SimEngineLight, SpOffset, RegisterOffset
@@ -17,7 +18,12 @@ from ....exploration_techniques.slicecutor import Slicecutor
 from ....exploration_techniques.explorer import Explorer
 from ....utils.constants import DEFAULT_STATEMENT
 from .resolver import IndirectJumpResolver
+from ....misc.ux import once
 
+try:
+    from ....engines import pcode
+except ImportError:
+    pcode = None
 
 l = logging.getLogger(name=__name__)
 
@@ -492,6 +498,7 @@ class BSSHook:
         for i in range(concrete_write_addr, concrete_write_length):
             self._written_addrs.add(i)
 
+
 #
 # Main class
 #
@@ -519,6 +526,10 @@ class JumpTableResolver(IndirectJumpResolver):
         self._find_bss_region()
 
     def filter(self, cfg, addr, func_addr, block, jumpkind):
+        if pcode is not None and isinstance(block.vex, pcode.lifter.IRSB):
+            if once('pcode__indirect_jump_resolver'):
+                l.warning('JumpTableResolver does not support P-Code IR yet; CFG may be incomplete.')
+            return False
 
         if is_arm_arch(self.project.arch):
             # For ARM, we support both jump tables and "call tables" (because of how crazy ARM compilers are...)
@@ -668,6 +679,10 @@ class JumpTableResolver(IndirectJumpResolver):
             init_registers_on_demand_bp = BP(when=BP_BEFORE, enabled=True, action=self._init_registers_on_demand)
             start_state.inspect.add_breakpoint('mem_read', init_registers_on_demand_bp)
 
+            # use Any as the concretization strategy
+            start_state.memory.read_strategies = [SimConcretizationStrategyAny()]
+            start_state.memory.write_strategies = [SimConcretizationStrategyAny()]
+
             # Create the slicecutor
             simgr = self.project.factory.simulation_manager(start_state, resilience=True)
             slicecutor = Slicecutor(annotatedcfg, force_taking_exit=True)
