@@ -48,18 +48,25 @@ class PTChunk(Chunk):
     def get_size(self):
         return self.state.memory.load(self.base + self._chunk_size_t_size, self._chunk_size_t_size) & ~CHUNK_FLAGS_MASK
 
+    def get_data_size(self):
+        chunk_size = self.get_size()
+        if self.is_free():
+            return chunk_size - 4 * self._chunk_size_t_size
+        else:
+            return chunk_size - 2 * self._chunk_size_t_size
+
     def _set_leading_size(self, size):
         level = silence_logger()
         chunk_flags = self.state.memory.load(self.base + self._chunk_size_t_size, self._chunk_size_t_size) \
                       & CHUNK_FLAGS_MASK
         unsilence_logger(level)
-        self.state.memory.store(self.base + self._chunk_size_t_size, size | chunk_flags)
+        self.state.memory.store(self.base + self._chunk_size_t_size, size | chunk_flags, size=self.state.arch.bytes)
 
     def _set_trailing_size(self, size):
         if self.is_free():
             next_chunk = self.next_chunk()
             if next_chunk is not None:
-                self.state.memory.store(next_chunk.base, size)
+                self.state.memory.store(next_chunk.base, size, self.state.arch.bytes)
 
     def set_size(self, size, is_free=None):  #pylint:disable=arguments-differ
         """
@@ -79,7 +86,7 @@ class PTChunk(Chunk):
                 self.heap._set_final_freeness(is_free)
         if is_free is not None and is_free or self.is_free():
             if next_chunk is not None:
-                self.state.memory.store(next_chunk.base, size)
+                self.state.memory.store(next_chunk.base, size, size=self.state.arch.bytes)
 
     def set_prev_freeness(self, is_free):
         """
@@ -91,9 +98,9 @@ class PTChunk(Chunk):
         size_field = self.state.memory.load(self.base + self._chunk_size_t_size, self._chunk_size_t_size)
         unsilence_logger(level)
         if is_free:
-            self.state.memory.store(self.base + self._chunk_size_t_size, size_field & ~CHUNK_P_MASK)
+            self.state.memory.store(self.base + self._chunk_size_t_size, size_field & ~CHUNK_P_MASK, size=self.state.arch.bytes)
         else:
-            self.state.memory.store(self.base + self._chunk_size_t_size, size_field | CHUNK_P_MASK)
+            self.state.memory.store(self.base + self._chunk_size_t_size, size_field | CHUNK_P_MASK, size=self.state.arch.bytes)
 
     def is_prev_free(self):
         """
@@ -177,13 +184,13 @@ class PTChunk(Chunk):
         :returns: If possible, the forward chunk; otherwise, raises an error
         """
         if self.is_free():
-            base = self.state.memory.load(self.base + 2 * self._chunk_size_t_size, self._chunk_size_t_size)
+            base = self.state.memory.load(self.base + 2 * self._chunk_size_t_size, self._chunk_size_t_size, endness=self.state.arch.memory_endness)
             return PTChunk(base, self.state)
         else:
             raise SimHeapError("Attempted to access the forward chunk of an allocated chunk")
 
     def set_fwd_chunk(self, fwd):
-        self.state.memory.store(self.base + 2 * self._chunk_size_t_size, fwd.base)
+        self.state.memory.store(self.base + 2 * self._chunk_size_t_size, fwd.base, endness=self.state.arch.memory_endness, size=self.state.arch.bytes)
 
     def bck_chunk(self):
         """
@@ -193,13 +200,13 @@ class PTChunk(Chunk):
         :returns: If possible, the backward chunk; otherwise, raises an error
         """
         if self.is_free():
-            base = self.state.memory.load(self.base + 3 * self._chunk_size_t_size, self._chunk_size_t_size)
+            base = self.state.memory.load(self.base + 3 * self._chunk_size_t_size, self._chunk_size_t_size, endness=self.state.arch.memory_endness)
             return PTChunk(base, self.state)
         else:
             raise SimHeapError("Attempted to access the backward chunk of an allocated chunk")
 
     def set_bck_chunk(self, bck):
-        self.state.memory.store(self.base + 3 * self._chunk_size_t_size, bck.base)
+        self.state.memory.store(self.base + 3 * self._chunk_size_t_size, bck.base, endness=self.state.arch.memory_endness, size=self.state.arch.bytes)
 
 class PTChunkIterator:
     def __init__(self, chunk, cond=lambda chunk: True):
@@ -234,6 +241,8 @@ class SimHeapPTMalloc(SimHeapFreelist):
     (corresponding to their different sizes), this more basic model uses a single list of chunks and searches for free
     chunks using a first-fit algorithm.
 
+    **NOTE:** The plugin must be registered using ``register_plugin`` with name ``heap`` in order to function properly.
+
     :ivar heap_base: the address of the base of the heap in memory
     :ivar heap_size: the total size of the main memory region managed by the heap in memory
     :ivar mmap_base: the address of the region from which large mmap allocations will be made
@@ -307,9 +316,9 @@ class SimHeapPTMalloc(SimHeapFreelist):
         this. Nonetheless, for now it is implemented as if an additional chunk followed the final chunk.
         """
         if flag:
-            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, ~CHUNK_P_MASK)
+            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, ~CHUNK_P_MASK, size=self.state.arch.bytes)
         else:
-            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, CHUNK_P_MASK)
+            self.state.memory.store(self.heap_base + self.heap_size - self._chunk_size_t_size, CHUNK_P_MASK, size=self.state.arch.bytes)
 
     def _make_chunk_size(self, req_size):
         """
@@ -598,7 +607,7 @@ class SimHeapPTMalloc(SimHeapFreelist):
         if not self._initialized:
             self.state.memory.store(self.free_head_chunk.base + self._chunk_size_t_size
                                     , ((self.heap_size - 2 * self._chunk_size_t_size) & ~self._chunk_align_mask)
-                                    | CHUNK_P_MASK)
+                                    | CHUNK_P_MASK, size=self.state.arch.bytes)
             self._set_final_freeness(True)
             self.free_head_chunk.set_fwd_chunk(self.free_head_chunk)
             self.free_head_chunk.set_bck_chunk(self.free_head_chunk)
